@@ -1,7 +1,7 @@
 ////////////////////////////////////////////////////////////////////////////
 //                           **** WAVPACK ****                            //
 //                  Hybrid Lossless Wavefile Compressor                   //
-//                Copyright (c) 1998 - 2019 David Bryant.                 //
+//                Copyright (c) 1998 - 2020 David Bryant.                 //
 //                          All Rights Reserved.                          //
 //      Distributed under the BSD Software License (see license.txt)      //
 ////////////////////////////////////////////////////////////////////////////
@@ -29,7 +29,7 @@
 
 static const char *sign_on = "\n"
 " WVTEST  libwavpack Tester/Exerciser for WavPack  %s Version %s\n"
-" Copyright (c) 2019 David Bryant.  All Rights Reserved.\n\n";
+" Copyright (c) 2020 David Bryant.  All Rights Reserved.\n\n";
 
 static const char *version_warning = "\n"
 " WARNING: WVTEST using libwavpack version %s, expected %s (see README)\n\n";
@@ -1354,7 +1354,7 @@ static void float_to_integer_samples (float *samples, int num_samples, int bits)
         else
             isample = floor (*samples * scalar);
 
-        *(int32_t *)samples = isample << ishift;
+        *(int32_t *)samples = (uint32_t) isample << ishift;
         samples++;
     } 
 }
@@ -1381,10 +1381,7 @@ static void float_to_32bit_integer_samples (float *samples, int num_samples)
                 tzeros++;
 
             while (tzeros--)
-                if (frandom() > 0.5)
-                    isample = (isample << 1) + 1;
-                else
-                    isample <<= 1;
+                isample = ((unsigned int) isample << 1) + ((frandom() > 0.5) ? 1 : 0);
         }
 
         *(int32_t *)samples = isample;
