@@ -20,11 +20,11 @@
 // This flag causes this module to take into account the size of the header
 // (which grows with more decorrelation passes) when making decisions about
 // adding additional passes (as opposed to just considering the resulting
-// magnitude of the residuals). With small blocks this seems to work correctly,
-// but with longer blocks it seems to actually hurt compression (for reasons I
-// cannot explain), so it's disabled by default
+// magnitude of the residuals). With really long blocks it seems to actually
+// hurt compression (for reasons I cannot explain), but with short blocks it
+// works okay, so we're enabling it for now.
 
-//#define USE_OVERHEAD
+#define USE_OVERHEAD
 
 // If the log2 value of any sample in a buffer being scanned exceeds this value,
 // we abandon that configuration. This prevents us from going down paths that
@@ -36,7 +36,7 @@
 
 #ifdef OPT_ASM_X86
     #define PACK_DECORR_MONO_PASS_CONT pack_decorr_mono_pass_cont_x86
-#elif defined(OPT_ASM_X64) && (defined (_WIN64) || defined(__CYGWIN__) || defined(__MINGW64__))
+#elif defined(OPT_ASM_X64) && (defined (_WIN64) || defined(__CYGWIN__) || defined(__MINGW64__) || defined(__midipix__))
     #define PACK_DECORR_MONO_PASS_CONT pack_decorr_mono_pass_cont_x64win
 #elif defined(OPT_ASM_X64)
     #define PACK_DECORR_MONO_PASS_CONT pack_decorr_mono_pass_cont_x64
@@ -79,7 +79,7 @@ static void decorr_mono_pass (int32_t *in_samples, int32_t *out_samples, uint32_
     dpp->weight_A = restore_weight (store_weight (dpp->weight_A));
 
     for (i = 0; i < 8; ++i)
-        dpp->samples_A [i] = exp2s (log2s (dpp->samples_A [i]));
+        dpp->samples_A [i] = wp_exp2s (wp_log2s (dpp->samples_A [i]));
 
     if (dpp->term > MAX_TERM) {
         while (num_samples--) {
