@@ -66,7 +66,7 @@ static uint32_t __inline read_code (Bitstream *bs, uint32_t maxcode);
 
 int32_t FASTCALL get_word (WavpackStream *wps, int chan, int32_t *correction)
 {
-    register struct entropy_data *c = wps->w.c + chan;
+    struct entropy_data *c = wps->w.c + chan;
     uint32_t ones_count, low, mid, high;
     int32_t value;
     int sign;
@@ -124,8 +124,8 @@ int32_t FASTCALL get_word (WavpackStream *wps, int chan, int32_t *correction)
             wps->wvbits.bc += sizeof (*(wps->wvbits.ptr)) * 8;
         }
 
-#ifdef _WIN32
-        _BitScanForward (&ones_count, ~wps->wvbits.sr);
+#ifdef _MSC_VER
+        { unsigned long res; _BitScanForward (&res, (unsigned long)~wps->wvbits.sr); ones_count = (uint32_t) res; }
 #else
         ones_count = __builtin_ctz (~wps->wvbits.sr);
 #endif
@@ -288,6 +288,10 @@ int32_t FASTCALL get_word (WavpackStream *wps, int chan, int32_t *correction)
 
     low &= 0x7fffffff;
     high &= 0x7fffffff;
+
+    if (low > high)         // make sure high and low make sense
+        high = low;
+
     mid = (high + low + 1) >> 1;
 
     if (!c->error_limit)
@@ -399,8 +403,8 @@ int32_t get_words_lossless (WavpackStream *wps, int32_t *buffer, int32_t nsample
             bs->bc += sizeof (*(bs->ptr)) * 8;
         }
 
-#ifdef _WIN32
-        _BitScanForward (&ones_count, ~wps->wvbits.sr);
+#ifdef _MSC_VER
+        { unsigned long res; _BitScanForward (&res, (unsigned long)~wps->wvbits.sr); ones_count = (uint32_t) res; }
 #else
         ones_count = __builtin_ctz (~wps->wvbits.sr);
 #endif
